# frozen_string_literal: true

module Chotto
  AND_CONJUCTION = :and
  OR_CONJUCTION = :or
  DEFAULT_CONJUCTION = AND_CONJUCTION
  OLDEST_FIRST_ORDER = 'oldest_first'
  NEWEST_FIRST_ORDER = 'newest_first'
  DEFAULT_ORDER = OLDEST_FIRST_ORDER

  class Messages
    attr_accessor :query, :current_conjuction, :token_count, :order, :only_new
    attr_reader :db

    include Enumerable

    def initialize(db:, only_new:)
      @db = db
      @current_conjuction = AND_CONJUCTION
      @token_count = 0
      @order = DEFAULT_ORDER
      @only_new = only_new
      clear_filter!
    end

    def or
      @current_conjuction = OR_CONJUCTION
      self
    end

    def and
      @current_conjuction = AND_CONJUCTION
      self
    end

    def filter(params)
      case params
      when String
        add_filter_from_string(params)
      when Hash
        add_filter_from_hash(params)
      end

      self
    end

    def clear_filter!
      @query = []
    end

    def newest_first
      @order = NEWEST_FIRST_ORDER

      self
    end

    def oldest_first
      @order = OLDEST_FIRST_ORDER

      self
    end

    def each
      db.search_messages(query_string).each do |msg|
        yield(
            Message.new(
              msg: msg,
              db: db
            ))
      end
    end

    def add_filter_from_string(string)
      @query << Token.new(current_conjuction, :direct, string, self)
    end

    def add_filter_from_hash(hash)
      group = TokenGroup.new(current_conjuction, [], self)
      hash.each do |key, value|
        group.push_token(key, value)
      end
      @query << group
    end

    def query_string
      string_from_filters = query.map(&:to_query).join(' ')
      if only_new
        "tag:new AND (#{string_from_filters})"
      else
        string_from_filters
      end
    end

    def next_token_id
      @token_count += 1
    end
  end
end
